@@ -539,7 +539,7 @@ get_info (DC240 * camera)
     {
       DBG (0,
 	   "Camera model (%d) is not DC-240 (5).  "
-	   "Only the DC-240 is supported by this driver.\n");
+	   "Only the DC-240 is supported by this driver.\n", camera->model);
     }
 
   camera->ver_major = buf[2];
@@ -710,7 +710,7 @@ end_of_data (SANE_Int fd)
 	}
       else
 	{
-	  DBG (127, "end_of_data: waiting...\n", c);
+	  DBG (127, "end_of_data: waiting...\n");
 	}
       sleep (1);		/* not too fast */
     }
@@ -1010,22 +1010,34 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 {
   SANE_Status status;
 
+  if (option < 0 || option >= NELEMS (sod))
+    return SANE_STATUS_INVAL;	/* Unknown option ... */
+
+  /* Need to put this DBG line after the range check on option */
   DBG (127, "control_option(handle=%p,opt=%s,act=%s,val=%p,info=%p)\n",
        handle, sod[option].title,
        (action ==
 	SANE_ACTION_SET_VALUE ? "SET" : (action ==
 					 SANE_ACTION_GET_VALUE ? "GET" :
-					 "SETAUTO")), value, info);
+					 "SETAUTO")), value, (void *) info);
 
   if (handle != MAGIC || !is_open)
     return SANE_STATUS_INVAL;	/* Unknown handle ... */
 
-  if (option < 0 || option >= NELEMS (sod))
-    return SANE_STATUS_INVAL;	/* Unknown option ... */
-
   switch (action)
     {
     case SANE_ACTION_SET_VALUE:
+
+      /* Can't set disabled options */
+      if (!SANE_OPTION_IS_ACTIVE (sod[option].cap))
+	{
+	  return (SANE_STATUS_INVAL);
+	}
+
+      /* initialize info to zero - we'll OR in various values later */
+      if (info)
+	*info = 0;
+
       status = sanei_constrain_value (sod + option, value, &myinfo);
       if (status != SANE_STATUS_GOOD)
 	{
@@ -1036,7 +1048,11 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
       switch (option)
 	{
 	case DC240_OPT_IMAGE_NUMBER:
-	  Camera.current_picture_number = *(SANE_Word *) value;
+	  if (*(SANE_Word *) value <= Camera.pic_taken)
+	    Camera.current_picture_number = *(SANE_Word *) value;
+	  else
+	    Camera.current_picture_number = Camera.pic_taken;
+
 	  myinfo |= SANE_INFO_RELOAD_PARAMS;
 
 	  /* get the image's resolution, unless the camera has no 
@@ -1051,6 +1067,8 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 
 	case DC240_OPT_THUMBS:
 	  dc240_opt_thumbnails = !!*(SANE_Word *) value;
+
+	  /* Thumbnail forces an image size change: */
 	  myinfo |= SANE_INFO_RELOAD_PARAMS;
 
 	  if (Camera.pic_taken != 0)
@@ -1061,7 +1079,20 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  break;
 
 	case DC240_OPT_SNAP:
-	  dc240_opt_snap = !!*(SANE_Word *) value;
+	  switch (*(SANE_Bool *) value)
+	    {
+	    case SANE_TRUE:
+	      dc240_opt_snap = SANE_TRUE;
+	      break;
+	    case SANE_FALSE:
+	      dc240_opt_snap = SANE_FALSE;
+	      break;
+	    default:
+	      return SANE_STATUS_INVAL;
+	    }
+
+	  /* Snap forces new image size and changes image range */
+
 	  myinfo |= SANE_INFO_RELOAD_PARAMS | SANE_INFO_RELOAD_OPTIONS;
 	  /* if we are snapping a new one */
 	  if (dc240_opt_snap)
@@ -1103,7 +1134,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  break;
 
 	case DC240_OPT_FOLDER:
-	  printf ("FIXME set folder not implemented yet\n");
+	  DBG (1, "FIXME set folder not implemented yet\n");
 	  break;
 
 	case DC240_OPT_DEFAULT:
@@ -1144,6 +1175,12 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
       break;
 
     case SANE_ACTION_GET_VALUE:
+      /* Can't return status for disabled options */
+      if (!SANE_OPTION_IS_ACTIVE (sod[option].cap))
+	{
+	  return (SANE_STATUS_INVAL);
+	}
+
       switch (option)
 	{
 	case 0:
@@ -1191,7 +1228,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	}
     }
 
-  if (info)
+  if (info && action == SANE_ACTION_SET_VALUE)
     {
       *info = myinfo;
       myinfo = 0;
@@ -1265,6 +1302,11 @@ METHODDEF (void) jpeg_skip_input_data (j_decompress_ptr cinfo, long num_bytes)
   src->pub.bytes_in_buffer -= (size_t) num_bytes;
 }
 
+static SANE_Byte linebuffer[HIGHRES_WIDTH * 3];
+static SANE_Int linebuffer_size = 0;
+static SANE_Int linebuffer_index = 0;
+
+
 METHODDEF (void)
 jpeg_term_source (j_decompress_ptr UNUSEDARG cinfo)
 {
@@ -1408,6 +1450,8 @@ sane_start (SANE_Handle handle)
     (void) jpeg_start_decompress (&cinfo);
     row_stride = cinfo.output_width * cinfo.output_components;
 
+    linebuffer_size = 0;
+    linebuffer_index = 0;
   }
 
   Camera.scanning = SANE_TRUE;	/* don't overlap scan requests */
@@ -1415,10 +1459,6 @@ sane_start (SANE_Handle handle)
   return SANE_STATUS_GOOD;
 }
 
-static SANE_Byte linebuffer[HIGHRES_WIDTH * 3];
-static SANE_Int linebuffer_size = 0;
-static SANE_Int linebuffer_index = 0;
-
 SANE_Status
 sane_read (SANE_Handle UNUSEDARG handle, SANE_Byte * data,
 	   SANE_Int max_length, SANE_Int * length)
@@ -1426,6 +1466,11 @@ sane_read (SANE_Handle UNUSEDARG handle, SANE_Byte * data,
   SANE_Int lines = 0;
   SANE_Char filename_buf[256];
 
+  if (Camera.scanning == SANE_FALSE)
+    {
+      return SANE_STATUS_INVAL;
+    }
+
   /* If there is anything in the buffer, satisfy the read from there */
   if (linebuffer_size && linebuffer_index < linebuffer_size)
     {
@@ -1443,6 +1488,8 @@ sane_read (SANE_Handle UNUSEDARG handle, SANE_Byte * data,
 
   if (cinfo.output_scanline >= cinfo.output_height)
     {
+      *length = 0;
+
       /* clean up comms with the camera */
       if (end_of_data (Camera.fd) == -1)
 	{
@@ -1463,9 +1510,10 @@ sane_read (SANE_Handle UNUSEDARG handle, SANE_Byte * data,
 	  image_range.max--;
 
 	  myinfo |= SANE_INFO_RELOAD_OPTIONS | SANE_INFO_RELOAD_PARAMS;
-	  strcpy((char *)filename_buf, strrchr((char *)name_buf+1,'\\')+1 );
-	  strcpy(strrchr((char *)filename_buf,'.'),"JPG");
-	  dir_delete ((SANE_String) filename_buf );
+	  strcpy ((char *) filename_buf,
+		  strrchr ((char *) name_buf + 1, '\\') + 1);
+	  strcpy (strrchr ((char *) filename_buf, '.'), "JPG");
+	  dir_delete ((SANE_String) filename_buf);
 
 	}
       if (dc240_opt_autoinc)
@@ -1508,8 +1556,36 @@ sane_read (SANE_Handle UNUSEDARG handle, SANE_Byte * data,
 void
 sane_cancel (SANE_Handle UNUSEDARG handle)
 {
+  unsigned char cancel_byte[] = { 0xe4 };
+
   if (Camera.scanning)
     {
+
+      /* Flush any pending data from the camera before continuing */
+      {
+	SANE_Int n;
+	SANE_Char flush[1024];
+	do
+	  {
+	    sleep (1);
+	    n = read (Camera.fd, flush, 1024);
+	    if (n > 0)
+	      {
+		DBG (127, "%s: flushed %d bytes\n", "sane_cancel", n);
+	      }
+	    else
+	      {
+		DBG (127, "%s: nothing to flush\n", "sane_cancel");
+	      }
+	  }
+	while (n > 0);
+      }
+
+      if (cinfo.output_scanline < cinfo.output_height)
+	{
+	  write (Camera.fd, cancel_byte, 1);
+	}
+
       Camera.scanning = SANE_FALSE;	/* done with scan */
     }
   else
@@ -1520,7 +1596,23 @@ SANE_Status
 sane_set_io_mode (SANE_Handle UNUSEDARG handle, SANE_Bool
 		  UNUSEDARG non_blocking)
 {
-  return SANE_STATUS_UNSUPPORTED;
+  /* sane_set_io_mode() is only valid during a scan */
+  if (Camera.scanning)
+    {
+      if (non_blocking == SANE_FALSE)
+	{
+	  return SANE_STATUS_GOOD;
+	}
+      else
+	{
+	  return SANE_STATUS_UNSUPPORTED;
+	}
+    }
+  else
+    {
+      /* We aren't currently scanning */
+      return SANE_STATUS_INVAL;
+    }
 }
 
 SANE_Status
@@ -1797,7 +1889,7 @@ read_dir (SANE_String dir)
 
       if (dir_insert (&dir_buf.entry[i]) != 0)
 	{
-	  DBG (1, "%s: error: failed to insert dir entry\n");
+	  DBG (1, "%s: error: failed to insert dir entry\n", f);
 	  return -1;
 	}
       retval++;
