@@ -483,7 +483,7 @@ init_gphoto2 (void)
 	{
 	  DBG (0,
 	       "%s: error: %d is not a valid speed for this camers.  Use \"gphoto2 --camera \"%s\" --abilities\" for list.\n",
-	       "init_gphoto2", Cam_data.camera_name, Cam_data.speed);
+	       "init_gphoto2", Cam_data.speed, Cam_data.camera_name);
 	  return SANE_STATUS_INVAL;
 	}
     }
@@ -518,6 +518,7 @@ close_gphoto2 (void)
       DBG (1, "close_gphoto2: error: could not close device\n");
     }
 
+  camera = NULL;
   close (hack_fd);
 }
 
@@ -661,7 +662,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
   DBG_INIT ();
 
   DBG (1,
-       "GPHOTO2 Backend $Id: gphoto2.c,v 1.20 2002/01/20 05:31:06 psfales Exp $\n");
+       "GPHOTO2 Backend $Id: gphoto2.c,v 1.23 2002/04/27 03:57:12 psfales Exp $\n");
 
   if (getenv ("GP_DEBUG"))
     {
@@ -909,11 +910,12 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
 }
 
 /*
- * sane_exit() - Required by SANE API, but otherwise not used
+ * sane_exit() - Required by SANE API.
  */
 void
 sane_exit (void)
 {
+  close_gphoto2 ();
 }
 
 /*
@@ -1012,12 +1014,16 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 {
   SANE_Status status;
 
+  if (option < 0 || option >= NELEMS (sod))
+    return SANE_STATUS_INVAL;	/* Unknown option ... */
+
+  /* Need to put this DBG line after the range check on option */
   DBG (127, "control_option(handle=%p,opt=%s,act=%s,val=%p,info=%p)\n",
        handle, sod[option].title,
        (action ==
 	SANE_ACTION_SET_VALUE ? "SET" : (action ==
 					 SANE_ACTION_GET_VALUE ? "GET" :
-					 "SETAUTO")), value, info);
+					 "SETAUTO")), value, (void *) info);
 
   if (handle != MAGIC || !is_open)
     return SANE_STATUS_INVAL;	/* Unknown handle ... */
@@ -1028,6 +1034,17 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
   switch (action)
     {
     case SANE_ACTION_SET_VALUE:
+
+      /* Can't set disabled options */
+      if (!SANE_OPTION_IS_ACTIVE (sod[option].cap))
+	{
+	  return (SANE_STATUS_INVAL);
+	}
+
+      /* initialize info to zero - we'll OR in various values later */
+      if (info)
+	*info = 0;
+
       status = sanei_constrain_value (sod + option, value, &myinfo);
       if (status != SANE_STATUS_GOOD)
 	{
@@ -1043,6 +1060,10 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  else
 	    Cam_data.current_picture_number = Cam_data.pic_taken;
 
+	  /* 
+	   * Setting a new image number could change image size (if
+	   * we supported that - which we hope to do someday!
+	   */
 	  myinfo |= SANE_INFO_RELOAD_PARAMS;
 
 	  /* get the image's resolution, unless the camera has no 
@@ -1060,6 +1081,8 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 
 	case GPHOTO2_OPT_THUMBS:
 	  gphoto2_opt_thumbnails = !!*(SANE_Word *) value;
+
+	  /* Thumbnail forces an image size change: */
 	  myinfo |= SANE_INFO_RELOAD_PARAMS;
 
 	  if (Cam_data.pic_taken != 0)
@@ -1073,7 +1096,20 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  break;
 
 	case GPHOTO2_OPT_SNAP:
-	  gphoto2_opt_snap = !!*(SANE_Word *) value;
+	  switch (*(SANE_Bool *) value)
+	    {
+	    case SANE_TRUE:
+	      gphoto2_opt_snap = SANE_TRUE;
+	      break;
+	    case SANE_FALSE:
+	      gphoto2_opt_snap = SANE_FALSE;
+	      break;
+	    default:
+	      return SANE_STATUS_INVAL;
+	    }
+
+	  /* Snap forces new image size and changes image range */
+
 	  myinfo |= SANE_INFO_RELOAD_PARAMS | SANE_INFO_RELOAD_OPTIONS;
 	  /* if we are snapping a new one */
 	  if (gphoto2_opt_snap)
@@ -1101,6 +1137,8 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 
 	case GPHOTO2_OPT_LOWRES:
 	  gphoto2_opt_lowres = !!*(SANE_Word *) value;
+
+	  /* Lowres potentially changes image size */
 	  myinfo |= SANE_INFO_RELOAD_PARAMS;
 
 /* FIXME - change the number of pictures left depending on resolution
@@ -1119,7 +1157,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  break;
 
 	case GPHOTO2_OPT_FOLDER:
-	  printf ("FIXME set folder not implemented yet\n");
+	  DBG (1, "FIXME set folder not implemented yet\n");
 	  break;
 
 	case GPHOTO2_OPT_DEFAULT:
@@ -1160,6 +1198,13 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
       break;
 
     case SANE_ACTION_GET_VALUE:
+
+      /* Can't return status for disabled options */
+      if (!SANE_OPTION_IS_ACTIVE (sod[option].cap))
+	{
+	  return (SANE_STATUS_INVAL);
+	}
+
       switch (option)
 	{
 	case 0:
@@ -1213,7 +1258,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	}
     }
 
-  if (info)
+  if (info && action == SANE_ACTION_SET_VALUE)
     {
       *info = myinfo;
       myinfo = 0;
@@ -1373,7 +1418,9 @@ sane_start (SANE_Handle handle)
   CHECK_RET (gp_file_get_mime_type (data_file, &mime_type));
   if (strcmp (GP_MIME_JPEG, mime_type) != 0)
     {
-      DBG (0, "FIXME - Only jpeg files currently supported\n");
+      DBG (0,
+	   "FIXME - Only jpeg files currently supported, can't do %s for file %s/%s\n",
+	   mime_type, cmdbuf, filename);
       return SANE_STATUS_INVAL;
     }
 
@@ -1412,6 +1459,11 @@ SANE_Status
 sane_read (SANE_Handle UNUSEDARG handle, SANE_Byte * data,
 	   SANE_Int max_length, SANE_Int * length)
 {
+  if (Cam_data.scanning == SANE_FALSE)
+    {
+      return SANE_STATUS_INVAL;
+    }
+
   /* If there is anything in the buffer, satisfy the read from there */
   if (linebuffer_size && linebuffer_index < linebuffer_size)
     {
@@ -1475,7 +1527,23 @@ SANE_Status
 sane_set_io_mode (SANE_Handle UNUSEDARG handle, SANE_Bool
 		  UNUSEDARG non_blocking)
 {
-  return SANE_STATUS_UNSUPPORTED;
+  /* sane_set_io_mode() is only valid during a scan */
+  if (Cam_data.scanning)
+    {
+      if (non_blocking == SANE_FALSE)
+	{
+	  return SANE_STATUS_GOOD;
+	}
+      else
+	{
+	  return SANE_STATUS_UNSUPPORTED;
+	}
+    }
+  else
+    {
+      /* We aren't currently scanning */
+      return SANE_STATUS_INVAL;
+    }
 }
 
 /*
@@ -1893,4 +1961,6 @@ converter_init (void)
   parms.pixels_per_line = cinfo.output_width;
   parms.lines = cinfo.output_height;
 
+  linebuffer_size = 0;
+  linebuffer_index = 0;
 }
