@@ -4,9 +4,7 @@
  * File:	 plustek-share.h - definitions for the backend and the driver
  *.............................................................................
  *
- * Copyright (C) 2000/2001 Gerhard Jaeger <g.jaeger@earthling.net>
- * Last Update:
- *		Gerhard Jaeger <g.jaeger@earthling.net>
+ * Copyright (C) 2000-2002 Gerhard Jaeger <gerhard@gjaeger.de>
  *.............................................................................
  * History:
  * 0.36 - initial version
@@ -21,6 +19,9 @@
  * 0.40 - added stuff to share with USB and Parport
  * 0.41 - changed the IOCTL version number
  *        added adjustment stuff
+ * 0.42 - added FLAG_CUSTOM_GAMMA and _MAP_ definitions
+ *        changed IOCTL interface to allow downloadable MAPS
+ *        added error codes
  *
  *.............................................................................
  *
@@ -106,13 +107,15 @@
 #define _PTDRV_CLOSE_DEVICE 	_IO('x',  9)				/* close 		 */
 #define _PTDRV_ACTION_BUTTON	_IOR('x', 10, int)	 		/* rd act. button*/
 #define _PTDRV_ADJUST           _IOR('x', 11, AdjDef)		/* adjust driver */
+#define _PTDRV_SETMAP           _IOR('x', 12, MapDef)		/* download gamma*/
 
 /*
  * this version MUST match the one inside the driver to make sure, that
  * both sides use the same structures. This version changes each time
  * the ioctl interface changes
  */
-#define _PTDRV_IOCTL_VERSION	0x0102
+#define _PTDRV_COMPAT_IOCTL_VERSION	0x0102
+#define _PTDRV_IOCTL_VERSION		0x0103
 
 /*.............................................................................
  * the structures for driver communication
@@ -190,11 +193,40 @@ typedef struct {
     unsigned short	wBeginY;		/* offset from top	*/
 } LensInfo, *pLensInfo;
 
+/**
+ * definition of gamma maps
+ */
+typedef struct {
+
+	int len;  		/**< gamma table len  */
+	int depth;  	/**< entry bit depth  */
+	int map_id;     /**< what map         */
+
+	void *map;      /**< pointer for map  */
+		
+} MapDef, *pMapDef;
+
+
 typedef struct {
 	int x;
 	int y;
 } OffsDef, *pOffsDef;
 
+/*
+ * for compatiblitiy to version 0x0102 drivers
+ */
+typedef struct {
+
+	int     lampOff;
+	int     lampOffOnEnd;
+	int     warmup;
+
+	OffsDef pos; 	/* for adjusting normal scan area       */
+	OffsDef tpa; 	/* for adjusting transparency scan area */
+	OffsDef neg; 	/* for adjusting negative scan area     */
+
+} CompatAdjDef, *pCompatAdjDef;
+
 /*
  * for adjusting the drivers
  */
@@ -203,9 +235,16 @@ typedef struct {
 	int     lampOffOnEnd;
 	int     warmup;
 
-	OffsDef pos; /* for adjusting normal scan area       */
-	OffsDef tpa; /* for adjusting transparency scan area */
-	OffsDef neg; /* for adjusting negative scan area     */
+	OffsDef pos; 	/* for adjusting normal scan area       */
+	OffsDef tpa; 	/* for adjusting transparency scan area */
+	OffsDef neg; 	/* for adjusting negative scan area     */
+	
+	/* for adjusting the default gamma settings */
+	double  rgamma;
+	double  ggamma;
+	double  bgamma;
+	
+	double  graygamma;
 
 } AdjDef, *pAdjDef;
 
@@ -287,6 +326,8 @@ typedef struct {
 #define SFLAG_TPA       	    0x00000080	/* has transparency	adapter     */
 #define SFLAG_BUTTONOPT         0x00000100  /* has buttons                  */
 
+#define SFLAG_CUSTOM_GAMMA		0x00000200  /* driver supports custom gamma */
+
 /*
  * (1.2.1) Provide the scanner ID. This field is valid when the wIOBase
  * field of ScannerCaps is not _NO_BASE
@@ -413,7 +454,7 @@ typedef struct {
 #define COLOR_GRAY16		COLOR_HALFTONE
 
 
-/* IDs the ASIC return */
+/* IDs the ASIC returns */
 #define _ASIC_IS_96001		0x0f	/* value for 96001	*/
 #define _ASIC_IS_96003		0x10	/* value for 96003  */
 #define _ASIC_IS_98001		0x81	/* value for 98001	*/
@@ -446,6 +487,47 @@ typedef struct {
  */
 #define _VAR_NOT_USED(x)	((x)=(x))
 
+/*
+ * for Gamma tables
+ */
+#define _MAP_RED    0
+#define _MAP_GREEN  1
+#define _MAP_BLUE   2
+#define _MAP_MASTER 3
+
+/*
+ * generic error codes...
+ */
+#define _OK			  0
+
+#define _FIRST_ERR	-9000
+
+#define _E_INIT	 	  (_FIRST_ERR-1)	/* already initialized				*/
+#define _E_NOT_INIT	  (_FIRST_ERR-2)	/* not initialized					*/
+#define _E_NULLPTR	  (_FIRST_ERR-3)	/* internal NULL-PTR detected		*/
+#define _E_ALLOC	  (_FIRST_ERR-4)	/* error allocating memory			*/
+#define _E_TIMEOUT	  (_FIRST_ERR-5)	/* signals a timeout condition		*/
+#define _E_INVALID	  (_FIRST_ERR-6)	/* invalid parameter detected		*/
+#define _E_INTERNAL	  (_FIRST_ERR-7)	/* internal error					*/
+#define _E_BUSY		  (_FIRST_ERR-8)	/* device is already in use			*/
+#define _E_ABORT	  (_FIRST_ERR-9)	/* operation aborted				*/
+#define	_E_LOCK		  (_FIRST_ERR-10)	/* cant lock resource				*/
+#define _E_NOSUPP	  (_FIRST_ERR-11)	/* feature or device not supported  */
+#define _E_NORESOURCE (_FIRST_ERR-12)	/* out of memo, resource busy...    */
+#define _E_VERSION	  (_FIRST_ERR-19)	/* version conflict					*/
+#define _E_NO_DEV	  (_FIRST_ERR-20)	/* device does not exist			*/
+#define _E_NO_CONN	  (_FIRST_ERR-21)	/* nothing connected				*/
+#define _E_PORTSEARCH (_FIRST_ERR-22)	/* parport_enumerate failed			*/
+#define _E_NO_PORT	  (_FIRST_ERR-23)	/* requested port does not exist	*/
+#define _E_REGISTER	  (_FIRST_ERR-24)	/* cannot register this device		*/
+#define _E_SEQUENCE	  (_FIRST_ERR-30)	/* caller sequence does not match	*/
+#define _E_NO_ASIC	  (_FIRST_ERR-31)	/* cant detect ASIC            	*/
+
+#define _E_LAMP_NOT_IN_POS	(_FIRST_ERR-40)
+#define _E_LAMP_NOT_STABLE	(_FIRST_ERR-41)
+#define _E_NODATA           (_FIRST_ERR-42)
+#define _E_BUFFER_TOO_SMALL (_FIRST_ERR-43)
+#define _E_DATAREAD         (_FIRST_ERR-44)
 
 /*
  * stuff needed for user space stuff
