@@ -47,7 +47,7 @@
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
 
-/* $Id: snapscan-scsi.c,v 1.11 2002/01/23 20:50:32 oliverschwartz Exp $
+/* $Id: snapscan-scsi.c,v 1.17 2002/05/02 17:19:15 oliverschwartz Exp $
    SnapScan backend scsi command functions */
 
 /* scanner scsi commands */
@@ -239,7 +239,9 @@ static SANE_Status open_scanner (SnapScan_Scanner *pss)
 
 static void close_scanner (SnapScan_Scanner *pss)
 {
-    DBG (DL_CALL_TRACE, "close_scanner\n");
+    static char me[] = "close_scanner";
+
+    DBG (DL_CALL_TRACE, "%s\n", me);
     if (pss->opens)
     {
         pss->opens--;
@@ -253,6 +255,8 @@ static void close_scanner (SnapScan_Scanner *pss)
             {
                 snapscani_usb_close (pss->fd);
             }
+        } else {
+            DBG(DL_INFO, "%s: handles left: %d\n,",me, pss->opens);
         }
     }
 }
@@ -545,7 +549,7 @@ static SANE_Status inquiry (SnapScan_Scanner *pss)
          "%s: effective buffer size = %lu bytes, %lu lines\n",
          me,
         (u_long) pss->buf_sz,
-        (u_long) (pss->buf_sz ? pss->buf_sz / pss->lines : 0));
+        (u_long) (pss->lines ? pss->buf_sz / pss->lines : 0));
     DBG (DL_DATA_TRACE,
         "%s: expected total scan data: %lu bytes\n",
         me,
@@ -772,30 +776,23 @@ static SANE_Status set_window (SnapScan_Scanner *pss)
             (int) (pss->actual_res*IN_PER_MM*SANE_UNFIX(pss->brx));
         int bryp =
             (int) (pss->actual_res*IN_PER_MM*SANE_UNFIX(pss->bry));
-        int tmp;
 
-        /* we don't guard against brx < tlx and bry < tly in the options */
-        if (brxp < tlxp)
-        {
-            tmp = tlxp;
-            tlxp = brxp;
-            brxp = tmp;
+        /* Check for brx > tlx and bry > tly */
+        if (brxp <= tlxp) {
+            tlxp = MAX (0, brxp - 75);
+        }
+        if (bryp <= tlyp) {
+            tlyp = MAX (0, bryp - 75);
         }
 
-        if (bryp < tlyp)
-        {
-            tmp = tlyp;
-            tlyp = bryp;
-            bryp = tmp;
-        }
         u_int_to_u_char4p (tlxp, pc + SET_WINDOW_P_TLX);
         u_int_to_u_char4p (tlyp, pc + SET_WINDOW_P_TLY);
         u_int_to_u_char4p (MAX (((unsigned) (brxp - tlxp)), 75),
                            pc + SET_WINDOW_P_WIDTH);
         u_int_to_u_char4p (MAX (((unsigned) (bryp - tlyp)), 75),
                            pc + SET_WINDOW_P_LENGTH);
-        DBG (DL_CALL_TRACE, "%s Width:  %d\n", me, brxp-tlxp);
-        DBG (DL_CALL_TRACE, "%s Length: %d\n", me, bryp-tlyp);
+        DBG (DL_INFO, "%s Width:  %d\n", me, brxp-tlxp);
+        DBG (DL_INFO, "%s Length: %d\n", me, bryp-tlyp);
     }
     pc[SET_WINDOW_P_BRIGHTNESS] = 128;
     pc[SET_WINDOW_P_THRESHOLD] =
@@ -809,7 +806,7 @@ static SANE_Status set_window (SnapScan_Scanner *pss)
             mode = pss->preview_mode;
 
         bpp = pss->pdev->depths[mode];
-        DBG (DL_CALL_TRACE, "%s Mode: %d\n", me, mode);
+        DBG (DL_MINOR_INFO, "%s Mode: %d\n", me, mode);
         switch (mode)
         {
         case MD_COLOUR:
@@ -835,7 +832,7 @@ static SANE_Status set_window (SnapScan_Scanner *pss)
         }
 
         pc[SET_WINDOW_P_BITS_PER_PIX] = bpp;
-        DBG (DL_DATA_TRACE, "%s: bits-per-pixel set to %d\n", me, (int) bpp);
+        DBG (DL_INFO, "%s: bits-per-pixel set to %d\n", me, (int) bpp);
     }
     /* the RIF bit is the high bit of the padding type */
     pc[SET_WINDOW_P_PADDING_TYPE] = 0x00 /*| (pss->negative ? 0x00 : 0x80) */ ;
@@ -870,13 +867,15 @@ static SANE_Status set_window (SnapScan_Scanner *pss)
     if (pss->source == SRC_TPO) {
         source |= 0x08;
     }
+    if (pss->source == SRC_ADF) {
+        source |= 0x10;
+    }
     pc[SET_WINDOW_P_OPERATION_MODE] = source;
-    DBG (DL_DATA_TRACE, "%s: operation mode set to %d\n", me, (int) source);
+    DBG (DL_MINOR_INFO, "%s: operation mode set to %d\n", me, (int) source);
     pc[SET_WINDOW_P_RED_UNDER_COLOR] = 0xff;    /* defaults */
     pc[SET_WINDOW_P_BLUE_UNDER_COLOR] = 0xff;
     pc[SET_WINDOW_P_GREEN_UNDER_COLOR] = 0xff;
 
-    DBG (DL_CALL_TRACE, "%s\n", me);
     status = snapscan_cmd (pss->pdev->bus, pss->fd, pss->cmd,
                SET_WINDOW_TOTAL_LEN, NULL, NULL);
     CHECK_STATUS (status, me, "snapscan_cmd");
@@ -921,6 +920,7 @@ static SANE_Status scsi_read (SnapScan_Scanner *pss, u_char read_type)
     return status;
 }
 
+/*
 static SANE_Status request_sense (SnapScan_Scanner *pss)
 {
     static const char *me = "request_sense";
@@ -945,6 +945,7 @@ static SANE_Status request_sense (SnapScan_Scanner *pss)
     }
     return status;
 }
+*/
 
 static SANE_Status send_diagnostic (SnapScan_Scanner *pss)
 {
@@ -952,9 +953,11 @@ static SANE_Status send_diagnostic (SnapScan_Scanner *pss)
     u_char cmd[] = {SEND_DIAGNOSTIC, 0x04, 0, 0, 0, 0};    /* self-test */
     SANE_Status status;
 
-    if (pss->pdev->model == PRISA620S    /* GP added */
+    if (pss->pdev->model == PRISA620S
         ||
-        pss->pdev->model == VUEGO610S)    /* SJU added */
+	pss->pdev->model == VUEGO610S
+	||
+	pss->pdev->model == SNAPSCAN1236) 
     {
         return SANE_STATUS_GOOD;
     }
@@ -973,7 +976,10 @@ static SANE_Status wait_scanner_ready (SnapScan_Scanner *pss)
 
     DBG (DL_CALL_TRACE, "%s\n", me);
 
-    for (retries = 5; retries; retries--)
+    /* if the tray is returning to the start position
+       no time to wait is returned by the scanner. We'll
+       try several times and sleep 1 second between each try. */
+    for (retries = 20; retries; retries--)
     {
         status = test_unit_ready (pss);
         switch (status)
@@ -994,6 +1000,9 @@ static SANE_Status wait_scanner_ready (SnapScan_Scanner *pss)
             /* hardware error; bail */
             DBG (DL_MAJOR_ERROR, "%s: hardware error detected.\n", me);
             return status;
+        case SANE_STATUS_JAMMED:
+        case SANE_STATUS_NO_DOCS:
+            return status;
         default:
             DBG (DL_MAJOR_ERROR,
                 "%s: unhandled request_sense result; trying again.\n",
@@ -1044,7 +1053,7 @@ static SANE_Status calibrate (SnapScan_Scanner *pss)
     SANE_Status status;
     int line_length = calibration_line_length(pss);
 
-    if (pss->hconfig & HCFG_CAL_ALLOWED) {
+    if ((pss->hconfig & HCFG_CAL_ALLOWED) && line_length) {
         int num_lines = pss->phys_buf_sz / line_length;
         if (num_lines > NUM_CALIBRATION_LINES)
         {
@@ -1080,62 +1089,14 @@ static SANE_Status calibrate (SnapScan_Scanner *pss)
     return SANE_STATUS_GOOD;
 }
 
-static SANE_Status get_firmware_name(char** firmware_path)
-{
-    char line[PATH_MAX];
-    char found = 0;
-    FILE *fp;
-    SANE_Status status = SANE_STATUS_GOOD;
-    char* me = "get_firmware_name";
-
-    fp = sanei_config_open (SNAPSCAN_CONFIG_FILE);
-    if (!fp)
-    {
-        DBG (0,
-             "%s: configuration file not found.\n",
-             me);
-        status = SANE_STATUS_INVAL;
-    }
-    else
-    {
-        while (sanei_config_read (line, sizeof (line), fp))
-        {
-            if (line[0] == '#')    /* ignore line comments */
-                continue;
-            if (!strlen(line))
-                continue;        /* ignore empty lines */
-            if (strncasecmp(line, FIRMWARE_KW, strlen(FIRMWARE_KW)) == 0)
-            {
-                found = 1;
-                break;
-            }
-        }
-        fclose (fp);
-    }
-    if (found)
-    {
-        sanei_config_get_string(line + strlen(FIRMWARE_KW), firmware_path);
-        if (*firmware_path == NULL)
-        {
-            DBG (0, "%s: Illegal firmware entry %s.\n", me, line);
-            status = SANE_STATUS_INVAL;
-        }
-    }
-    else
-    {
-        DBG (0, "%s: No firmware entry found in config file %s.\n", me, SNAPSCAN_CONFIG_FILE);
-        status = SANE_STATUS_INVAL;
-    }
-    return status;
-}
-
 static SANE_Status download_firmware(SnapScan_Scanner * pss)
 {
+    static const char *me = "download_firmware";
     unsigned char *pFwBuf, *pCDB;
-    char* firmware_path = NULL;
+    char* firmware = NULL;
     FILE *fd;
     size_t bufLength,cdbLength;
-    SANE_Status status;
+    SANE_Status status = SANE_STATUS_GOOD;
     char cModelName[8], cModel[255];
     unsigned char bModelNo;
     int readByte;
@@ -1144,15 +1105,26 @@ static SANE_Status download_firmware(SnapScan_Scanner * pss)
     zero_buf((unsigned char *)cModel, 255);
     sprintf(cModelName, "%d", bModelNo);
     DBG(DL_INFO, "Looking up %s\n", cModelName);
-    status = get_firmware_name(&firmware_path);
+    if (pss->pdev->firmware_filename) {
+        firmware = pss->pdev->firmware_filename;
+    } else if (default_firmware_filename) {
+        firmware = default_firmware_filename;
+    } else {
+        DBG (0,
+            "%s: No firmware entry found in config file %s.\n",
+            me,
+            SNAPSCAN_CONFIG_FILE
+        );
+        status = SANE_STATUS_INVAL;
+    }
     if (status == SANE_STATUS_GOOD)
     {
         cdbLength = 10;
-        DBG(DL_INFO, "Downloading %s\n", firmware_path);
-        fd = fopen(firmware_path,"r");
+        DBG(DL_INFO, "Downloading %s\n", firmware);
+        fd = fopen(firmware,"r");
         if(fd == NULL)
         {
-            DBG (0, "Cannot open firmware file %s\n", firmware_path);
+            DBG (0, "Cannot open firmware file %s\n", firmware);
             status = SANE_STATUS_INVAL;
         }
         else
@@ -1206,15 +1178,34 @@ static SANE_Status download_firmware(SnapScan_Scanner * pss)
             free(pCDB);
             fclose(fd);
         }
-        free(firmware_path);
     }
     return status;
 }
 
 /*
  * $Log: snapscan-scsi.c,v $
- * Revision 1.11  2002/01/23 20:50:32  oliverschwartz
- * Fix recognition of Acer 320U
+ * Revision 1.17  2002/05/02 17:19:15  oliverschwartz
+ * SnapScan backend 1.4.13: Support for ADF
+ *
+ * Revision 1.16  2002/04/27 15:35:17  oliverschwartz
+ * SnapScan backend 1.4.12: Fix option handling
+ *
+ * Revision 1.30  2002/04/27 14:41:22  oliverschwartz
+ * Print number of open handles in close_scanner()
+ *
+ * Revision 1.29  2002/04/10 21:46:48  oliverschwartz
+ * Removed illegal character
+ *
+ * Revision 1.28  2002/04/10 21:01:02  oliverschwartz
+ * Disable send_diagnostic() for 1236s
+ *
+ * Revision 1.27  2002/03/24 12:11:20  oliverschwartz
+ * Get name of firmware file in sane_init
+ *
+ * Revision 1.26  2002/01/23 20:42:41  oliverschwartz
+ * Improve recognition of Acer 320U
+ * Add sense_handler code for sense code 0x0b
+ * Fix for spaces in model strings
  *
  * Revision 1.25  2001/12/12 19:44:59  oliverschwartz
  * Clean up CVS log
