@@ -51,19 +51,25 @@
 #include <ctype.h>
 #include <errno.h>
 #include <limits.h>
-#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <math.h>
 
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
 
 #include "../include/sane/sane.h"
 #include "../include/sane/sanei.h"
 #include "../include/sane/saneopts.h"
+#include "../include/sane/sanei_config.h"
 
 #include "umax_pp_mid.h"
 #include "umax_pp.h"
@@ -71,7 +77,6 @@
 #define BACKEND_NAME	umax_pp
 #include "../include/sane/sanei_backend.h"
 
-#include "../include/sane/sanei_config.h"
 
 #ifdef DMALLOC
 #include "dmalloc.h"
@@ -99,7 +104,7 @@
 
 /* if you change the source, please set UMAX_PP_STATE to "devel". Do *not*
  * change the UMAX_PP_BUILD. */
-#define UMAX_PP_BUILD	6
+#define UMAX_PP_BUILD	8
 #define UMAX_PP_STATE	"devel"
 
 static int num_devices = 0;
@@ -109,8 +114,8 @@ static const SANE_Device **devarray = NULL;
 static Umax_PP_Device *first_dev = NULL;
 
 
-/* 1 Meg scan buffer */
-static long int buf_size = 2024 * 1024;
+/* 2 Meg scan buffer */
+static long int buf_size = 2048 * 1024;
 
 
 static int red_gain = 0;
@@ -162,7 +167,10 @@ static const SANE_Range u8_range = {
 
 #define UMAX_PP_DEFAULT_PORT		0x378
 
-
+/*
+ * devname may be either an hardware address for direct I/O (0x378 for instance)
+ * or the device name used by ppdev on linux systems        (/dev/parport0 )
+ */
 
 
 static SANE_Status
@@ -171,24 +179,48 @@ attach (const char *devname)
   Umax_PP_Descriptor *dev;
   int i;
   SANE_Status status = SANE_STATUS_GOOD;
-  int ret, prt, mdl;
+  int ret, prt = 0, mdl;
   char model[32];
+  char name[64];
 
+  memset (name, 0, 64);
 
-  if ((strlen (devname) < 3) || (strlen (devname) > 8))
+  if ((strlen (devname) < 3))
     return SANE_STATUS_INVAL;
 
-  if ((devname[0] == '0') && ((devname[1] == 'x') || (devname[1] == 'X')))
-    prt = strtol (devname + 2, NULL, 16);
-  else
-    prt = atoi (devname);
+  /* if the name begins with a slash, it's a device, else it's an addr */
+  if (devname != NULL)
+    {
+      if ((devname[0] == '/'))
+	{
+	  strncpy (name, devname, 64);
+	}
+      else
+	{
+	  if ((devname[0] == '0')
+	      && ((devname[1] == 'x') || (devname[1] == 'X')))
+	    prt = strtol (devname + 2, NULL, 16);
+	  else
+	    prt = atoi (devname);
+	}
+    }
 
 
   for (i = 0; i < num_devices; i++)
-    if (strcmp (devlist[i].port, devname) == 0)
-      return SANE_STATUS_GOOD;
+    {
+      if (devname[0] == '/')
+	{
+	  if (strcmp (devlist[i].ppdevice, devname) == 0)
+	    return SANE_STATUS_GOOD;
+	}
+      else
+	{
+	  if (strcmp (devlist[i].port, devname) == 0)
+	    return SANE_STATUS_GOOD;
+	}
+    }
 
-  ret = sanei_umax_pp_attach (prt);
+  ret = sanei_umax_pp_attach (prt, name);
   switch (ret)
     {
     case UMAX1220P_OK:
@@ -198,11 +230,11 @@ attach (const char *devname)
       status = SANE_STATUS_DEVICE_BUSY;
       break;
     case UMAX1220P_TRANSPORT_FAILED:
-      DBG (1, "attach: failed to init transport layer on port %s\n", devname);
+      DBG (1, "attach: failed to init transport layer on %s\n", devname);
       status = SANE_STATUS_IO_ERROR;
       break;
     case UMAX1220P_PROBE_FAILED:
-      DBG (1, "attach: failed to probe scanner on port %s\n", devname);
+      DBG (1, "attach: failed to probe scanner on %s\n", devname);
       status = SANE_STATUS_IO_ERROR;
       break;
     }
@@ -222,15 +254,14 @@ attach (const char *devname)
       ret = sanei_umax_pp_model (prt, &mdl);
       if (ret != UMAX1220P_OK)
 	{
-	  DBG (1, "attach: waiting for busy scanner on port %s\n", devname);
+	  DBG (1, "attach: waiting for busy scanner on %s\n", devname);
 	}
     }
   while (ret == UMAX1220P_BUSY);
 
   if (ret != UMAX1220P_OK)
     {
-      DBG (1, "attach: failed to recognize scanner model on port %s\n",
-	   devname);
+      DBG (1, "attach: failed to recognize scanner model on %s\n", devname);
       return SANE_STATUS_IO_ERROR;
     }
   sprintf (model, "Astra %dP", mdl);
@@ -260,7 +291,10 @@ attach (const char *devname)
   dev->sane.vendor = strdup ("UMAX");
   dev->sane.type = "flatbed scanner";
 
-  dev->port = strdup (devname);
+  if (devname[0] == '/')
+    dev->ppdevice = strdup (devname);
+  else
+    dev->port = strdup (devname);
   dev->buf_size = buf_size;
 
   if (mdl > 610)
@@ -312,7 +346,7 @@ init_options (Umax_PP_Device * dev)
 
   /* "Mode" group: */
 
-  dev->opt[OPT_MODE_GROUP].title = "Scan Mode";
+  dev->opt[OPT_MODE_GROUP].title = SANE_I18N("Scan Mode");
   dev->opt[OPT_MODE_GROUP].name = "";
   dev->opt[OPT_MODE_GROUP].desc = "";
   dev->opt[OPT_MODE_GROUP].type = SANE_TYPE_GROUP;
@@ -359,7 +393,7 @@ init_options (Umax_PP_Device * dev)
 
   /* "Geometry" group: */
 
-  dev->opt[OPT_GEOMETRY_GROUP].title = "Geometry";
+  dev->opt[OPT_GEOMETRY_GROUP].title = SANE_I18N("Geometry");
   dev->opt[OPT_GEOMETRY_GROUP].desc = "";
   dev->opt[OPT_GEOMETRY_GROUP].name = "";
   dev->opt[OPT_GEOMETRY_GROUP].type = SANE_TYPE_GROUP;
@@ -407,7 +441,7 @@ init_options (Umax_PP_Device * dev)
 
   /* "Enhancement" group: */
 
-  dev->opt[OPT_ENHANCEMENT_GROUP].title = "Enhancement";
+  dev->opt[OPT_ENHANCEMENT_GROUP].title = SANE_I18N("Enhancement");
   dev->opt[OPT_ENHANCEMENT_GROUP].desc = "";
   dev->opt[OPT_ENHANCEMENT_GROUP].name = "";
   dev->opt[OPT_ENHANCEMENT_GROUP].type = SANE_TYPE_GROUP;
@@ -415,9 +449,9 @@ init_options (Umax_PP_Device * dev)
   dev->opt[OPT_ENHANCEMENT_GROUP].constraint_type = SANE_CONSTRAINT_NONE;
 
   /* lamp control */
-  dev->opt[OPT_LAMP_CONTROL].name = "lamp-control";
-  dev->opt[OPT_LAMP_CONTROL].title = "Lamp on";
-  dev->opt[OPT_LAMP_CONTROL].desc = "Sets lamp on/off";
+  dev->opt[OPT_LAMP_CONTROL].name = SANE_I18N("lamp-control");
+  dev->opt[OPT_LAMP_CONTROL].title = SANE_I18N("Lamp on");
+  dev->opt[OPT_LAMP_CONTROL].desc = SANE_I18N("Sets lamp on/off");
   dev->opt[OPT_LAMP_CONTROL].type = SANE_TYPE_BOOL;
   dev->opt[OPT_LAMP_CONTROL].size = sizeof (SANE_Word);
   dev->opt[OPT_LAMP_CONTROL].unit = SANE_UNIT_NONE;
@@ -482,16 +516,16 @@ init_options (Umax_PP_Device * dev)
 
   /*  gain group */
   dev->opt[OPT_MANUAL_GAIN].name = "manual-channel-gain";
-  dev->opt[OPT_MANUAL_GAIN].title = "Gain";
-  dev->opt[OPT_MANUAL_GAIN].desc = "Color channels gain settings";
+  dev->opt[OPT_MANUAL_GAIN].title = SANE_I18N("Gain");
+  dev->opt[OPT_MANUAL_GAIN].desc = SANE_I18N("Color channels gain settings");
   dev->opt[OPT_MANUAL_GAIN].type = SANE_TYPE_BOOL;
   dev->opt[OPT_MANUAL_GAIN].cap |= SANE_CAP_ADVANCED;
   dev->val[OPT_MANUAL_GAIN].w = SANE_FALSE;
 
   /* gray gain */
   dev->opt[OPT_GRAY_GAIN].name = "gray-gain";
-  dev->opt[OPT_GRAY_GAIN].title = "Gray gain";
-  dev->opt[OPT_GRAY_GAIN].desc = "Sets gray channel gain";
+  dev->opt[OPT_GRAY_GAIN].title = SANE_I18N("Gray gain");
+  dev->opt[OPT_GRAY_GAIN].desc = SANE_I18N("Sets gray channel gain");
   dev->opt[OPT_GRAY_GAIN].type = SANE_TYPE_INT;
   dev->opt[OPT_GRAY_GAIN].cap |= SANE_CAP_INACTIVE | SANE_CAP_ADVANCED;
   dev->opt[OPT_GRAY_GAIN].unit = SANE_UNIT_NONE;
@@ -502,8 +536,8 @@ init_options (Umax_PP_Device * dev)
 
   /* red gain */
   dev->opt[OPT_RED_GAIN].name = "red-gain";
-  dev->opt[OPT_RED_GAIN].title = "Red gain";
-  dev->opt[OPT_RED_GAIN].desc = "Sets red channel gain";
+  dev->opt[OPT_RED_GAIN].title = SANE_I18N("Red gain");
+  dev->opt[OPT_RED_GAIN].desc = SANE_I18N("Sets red channel gain");
   dev->opt[OPT_RED_GAIN].type = SANE_TYPE_INT;
   dev->opt[OPT_RED_GAIN].cap |= SANE_CAP_INACTIVE | SANE_CAP_ADVANCED;
   dev->opt[OPT_RED_GAIN].unit = SANE_UNIT_NONE;
@@ -514,8 +548,8 @@ init_options (Umax_PP_Device * dev)
 
   /* green gain */
   dev->opt[OPT_GREEN_GAIN].name = "green-gain";
-  dev->opt[OPT_GREEN_GAIN].title = "Green gain";
-  dev->opt[OPT_GREEN_GAIN].desc = "Sets green channel gain";
+  dev->opt[OPT_GREEN_GAIN].title = SANE_I18N("Green gain");
+  dev->opt[OPT_GREEN_GAIN].desc = SANE_I18N("Sets green channel gain");
   dev->opt[OPT_GREEN_GAIN].type = SANE_TYPE_INT;
   dev->opt[OPT_GREEN_GAIN].cap |= SANE_CAP_INACTIVE | SANE_CAP_ADVANCED;
   dev->opt[OPT_GREEN_GAIN].unit = SANE_UNIT_NONE;
@@ -526,8 +560,8 @@ init_options (Umax_PP_Device * dev)
 
   /* blue gain */
   dev->opt[OPT_BLUE_GAIN].name = "blue-gain";
-  dev->opt[OPT_BLUE_GAIN].title = "Blue gain";
-  dev->opt[OPT_BLUE_GAIN].desc = "Sets blue channel gain";
+  dev->opt[OPT_BLUE_GAIN].title = SANE_I18N("Blue gain");
+  dev->opt[OPT_BLUE_GAIN].desc = SANE_I18N("Sets blue channel gain");
   dev->opt[OPT_BLUE_GAIN].type = SANE_TYPE_INT;
   dev->opt[OPT_BLUE_GAIN].cap |= SANE_CAP_INACTIVE | SANE_CAP_ADVANCED;
   dev->opt[OPT_BLUE_GAIN].unit = SANE_UNIT_NONE;
@@ -538,16 +572,16 @@ init_options (Umax_PP_Device * dev)
 
   /*  highlight group */
   dev->opt[OPT_MANUAL_HIGHLIGHT].name = "manual-highlight";
-  dev->opt[OPT_MANUAL_HIGHLIGHT].title = "Highlight";
-  dev->opt[OPT_MANUAL_HIGHLIGHT].desc = "Color channels highlight settings";
+  dev->opt[OPT_MANUAL_HIGHLIGHT].title = SANE_I18N("Highlight");
+  dev->opt[OPT_MANUAL_HIGHLIGHT].desc = SANE_I18N("Color channels highlight settings");
   dev->opt[OPT_MANUAL_HIGHLIGHT].type = SANE_TYPE_BOOL;
   dev->opt[OPT_MANUAL_HIGHLIGHT].cap |= SANE_CAP_ADVANCED;
   dev->val[OPT_MANUAL_HIGHLIGHT].w = SANE_FALSE;
 
   /* gray highlight */
   dev->opt[OPT_GRAY_HIGHLIGHT].name = "gray-highlight";
-  dev->opt[OPT_GRAY_HIGHLIGHT].title = "Gray highlight";
-  dev->opt[OPT_GRAY_HIGHLIGHT].desc = "Sets gray channel highlight";
+  dev->opt[OPT_GRAY_HIGHLIGHT].title = SANE_I18N("Gray highlight");
+  dev->opt[OPT_GRAY_HIGHLIGHT].desc = SANE_I18N("Sets gray channel highlight");
   dev->opt[OPT_GRAY_HIGHLIGHT].type = SANE_TYPE_INT;
   dev->opt[OPT_GRAY_HIGHLIGHT].cap |= SANE_CAP_INACTIVE | SANE_CAP_ADVANCED;
   dev->opt[OPT_GRAY_HIGHLIGHT].unit = SANE_UNIT_NONE;
@@ -558,8 +592,8 @@ init_options (Umax_PP_Device * dev)
 
   /* red highlight */
   dev->opt[OPT_RED_HIGHLIGHT].name = "red-highlight";
-  dev->opt[OPT_RED_HIGHLIGHT].title = "Red highlight";
-  dev->opt[OPT_RED_HIGHLIGHT].desc = "Sets red channel highlight";
+  dev->opt[OPT_RED_HIGHLIGHT].title = SANE_I18N("Red highlight");
+  dev->opt[OPT_RED_HIGHLIGHT].desc = SANE_I18N("Sets red channel highlight");
   dev->opt[OPT_RED_HIGHLIGHT].type = SANE_TYPE_INT;
   dev->opt[OPT_RED_HIGHLIGHT].cap |= SANE_CAP_INACTIVE | SANE_CAP_ADVANCED;
   dev->opt[OPT_RED_HIGHLIGHT].unit = SANE_UNIT_NONE;
@@ -570,8 +604,8 @@ init_options (Umax_PP_Device * dev)
 
   /* green highlight */
   dev->opt[OPT_GREEN_HIGHLIGHT].name = "green-highlight";
-  dev->opt[OPT_GREEN_HIGHLIGHT].title = "Green highlight";
-  dev->opt[OPT_GREEN_HIGHLIGHT].desc = "Sets green channel highlight";
+  dev->opt[OPT_GREEN_HIGHLIGHT].title = SANE_I18N("Green highlight");
+  dev->opt[OPT_GREEN_HIGHLIGHT].desc = SANE_I18N("Sets green channel highlight");
   dev->opt[OPT_GREEN_HIGHLIGHT].type = SANE_TYPE_INT;
   dev->opt[OPT_GREEN_HIGHLIGHT].cap |= SANE_CAP_INACTIVE | SANE_CAP_ADVANCED;
   dev->opt[OPT_GREEN_HIGHLIGHT].unit = SANE_UNIT_NONE;
@@ -582,8 +616,8 @@ init_options (Umax_PP_Device * dev)
 
   /* blue highlight */
   dev->opt[OPT_BLUE_HIGHLIGHT].name = "blue-highlight";
-  dev->opt[OPT_BLUE_HIGHLIGHT].title = "Blue highlight";
-  dev->opt[OPT_BLUE_HIGHLIGHT].desc = "Sets blue channel highlight";
+  dev->opt[OPT_BLUE_HIGHLIGHT].title = SANE_I18N("Blue highlight");
+  dev->opt[OPT_BLUE_HIGHLIGHT].desc = SANE_I18N("Sets blue channel highlight");
   dev->opt[OPT_BLUE_HIGHLIGHT].type = SANE_TYPE_INT;
   dev->opt[OPT_BLUE_HIGHLIGHT].cap |= SANE_CAP_INACTIVE | SANE_CAP_ADVANCED;
   dev->opt[OPT_BLUE_HIGHLIGHT].unit = SANE_UNIT_NONE;
@@ -646,6 +680,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
       if (strncmp (cp, "option", 6) == 0 && isspace (cp[6]))
 	{
 
+	  DBG (3, "init: evaluating option <%s>\n", cp);
 	  cp += 7;
 	  cp = sanei_config_skip_whitespace (cp);
 
@@ -680,12 +715,12 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 		  devlist[0].buf_size = val;
 		}
 	    }
-	  else if (strncmp (cp, "astra", 6) == 0)
+	  else if (strncmp (cp, "astra", 5) == 0)
 	    {
 	      char *end;
 	      long int val;
 
-	      cp += 7;
+	      cp += 6;
 
 	      errno = 0;
 	      val = strtol (cp, &end, 0);
@@ -980,7 +1015,8 @@ sane_open (SANE_String_Const devicename, SANE_Handle * handle)
   Umax_PP_Device *dev;
   Umax_PP_Descriptor *desc;
   int i, j;
-  int rc, prt;
+  int rc, prt = 0;
+  char *name = NULL;
 
   DBG (3, "open: device `%s'\n", devicename);
 
@@ -1004,14 +1040,24 @@ sane_open (SANE_String_Const devicename, SANE_Handle * handle)
 
       desc = &devlist[i];
 
-      if ((devlist[i].port[0] == '0')
-	  && ((devlist[i].port[1] == 'x') || (devlist[i].port[1] == 'X')))
-	prt = strtol (devlist[i].port + 2, NULL, 16);
+      if (devlist[i].ppdevice != NULL)
+	{
+	  if (devlist[i].ppdevice[0] == '/')
+	    {
+	      name = devlist[i].ppdevice;
+	    }
+	}
       else
-	prt = atoi (devlist[i].port);
-      DBG (64, "open: devlist[i].port='%s' -> port=0x%X\n", devlist[i].port,
-	   prt);
-      rc = sanei_umax_pp_open (prt);
+	{
+	  if ((devlist[i].port[0] == '0')
+	      && ((devlist[i].port[1] == 'x') || (devlist[i].port[1] == 'X')))
+	    prt = strtol (devlist[i].port + 2, NULL, 16);
+	  else
+	    prt = atoi (devlist[i].port);
+	  DBG (64, "open: devlist[i].port='%s' -> port=0x%X\n",
+	       devlist[i].port, prt);
+	}
+      rc = sanei_umax_pp_open (prt, name);
     }
   else
     {
@@ -1025,22 +1071,44 @@ sane_open (SANE_String_Const devicename, SANE_Handle * handle)
       DBG (3, "open: trying default device %s, port=%s\n",
 	   devlist[0].sane.name, devlist[0].port);
 
-      rc = sanei_umax_pp_open (atoi (devlist[0].port));
+      rc = sanei_umax_pp_open (atoi (devlist[0].port), NULL);
 
       desc = &devlist[0];
     }
   switch (rc)
     {
     case UMAX1220P_TRANSPORT_FAILED:
-      DBG (1, "failed to init transport layer on port 0x%03X\n",
-	   atoi (desc->port));
+      if (name == NULL)
+	{
+	  DBG (1, "failed to init transport layer on port 0x%03X\n",
+	       atoi (desc->port));
+	}
+      else
+	{
+	  DBG (1, "failed to init transport layer on device %s\n", name);
+	}
       return SANE_STATUS_IO_ERROR;
+
     case UMAX1220P_SCANNER_FAILED:
-      DBG (1, "failed to initialize scanner on port 0x%03X\n",
-	   atoi (desc->port));
+      if (name == NULL)
+	{
+	  DBG (1, "failed to initialize scanner on port 0x%03X\n",
+	       atoi (desc->port));
+	}
+      else
+	{
+	  DBG (1, "failed to initialize scanner on device %s\n", name);
+	}
       return SANE_STATUS_IO_ERROR;
     case UMAX1220P_BUSY:
-      DBG (1, "busy scanner on port 0x%03X\n", atoi (desc->port));
+      if (name == NULL)
+	{
+	  DBG (1, "busy scanner on port 0x%03X\n", atoi (desc->port));
+	}
+      else
+	{
+	  DBG (1, "busy scanner on device %s\n", name);
+	}
       return SANE_STATUS_DEVICE_BUSY;
     }
 
@@ -1234,7 +1302,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
       return SANE_STATUS_INVAL;
     }
 
-  DBG (6, "control_option: option <%s>, action ... ", dev->opt[option].name,
+  DBG (6, "control_option: option <%s>, action ... %d", dev->opt[option].name,
        action);
 
   if (action == SANE_ACTION_GET_VALUE)
@@ -1894,8 +1962,8 @@ sane_start (SANE_Handle handle)
 	   dev->TopX,
 	   dev->TopY,
 	   dev->BottomX - dev->TopX,
-	   dev->BottomY - dev->TopY, dev->dpi, dev->green_gain << 4,
-	   dev->BottomY - dev->TopY, dev->dpi, dev->green_highlight << 4);
+	   dev->BottomY - dev->TopY, dev->dpi, dev->gray_gain << 4,
+	   dev->gray_highlight << 4);
       rc = sanei_umax_pp_start (dev->TopX,
 				dev->TopY,
 				dev->BottomX - dev->TopX,
@@ -1993,7 +2061,7 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
       /* re order data into RGB */
       if (dev->color == UMAX_PP_MODE_COLOR)
 	{
-	  DBG (64, "sane_read: reordering %d bytes of data (lines=%d)\n",
+	  DBG (64, "sane_read: reordering %ld bytes of data (lines=%d)\n",
 	       length, nl);
 	  lbuf = (SANE_Byte *) malloc (dev->bufsize);
 	  if (lbuf == NULL)
@@ -2003,16 +2071,34 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 	      return SANE_STATUS_NO_MEM;
 	    }
 	  /* this loop will be optimized when everything else will be working */
-	  for (y = 0; y < nl; y++)
+	  if (sanei_umax_pp_getastra () == 1600)
 	    {
-	      for (x = 0; x < dev->tw; x++)
+	      for (y = 0; y < nl; y++)
 		{
-		  lbuf[x * dev->bpp + y * ll] =
-		    dev->buf[dev->bufread + x + y * ll + 2 * dev->tw];
-		  lbuf[x * dev->bpp + y * ll + 1] =
-		    dev->buf[dev->bufread + x + y * ll + dev->tw];
-		  lbuf[x * dev->bpp + y * ll + 2] =
-		    dev->buf[dev->bufread + x + y * ll];
+		  for (x = 0; x < dev->tw; x++)
+		    {
+		      lbuf[x * dev->bpp + y * ll] =
+			dev->buf[dev->bufread + x + y * ll + 2 * dev->tw];
+		      lbuf[x * dev->bpp + y * ll + 1] =
+			dev->buf[dev->bufread + x + y * ll];
+		      lbuf[x * dev->bpp + y * ll + 2] =
+			dev->buf[dev->bufread + x + y * ll + dev->tw];
+		    }
+		}
+	    }
+	  else
+	    {
+	      for (y = 0; y < nl; y++)
+		{
+		  for (x = 0; x < dev->tw; x++)
+		    {
+		      lbuf[x * dev->bpp + y * ll] =
+			dev->buf[dev->bufread + x + y * ll + 2 * dev->tw];
+		      lbuf[x * dev->bpp + y * ll + 1] =
+			dev->buf[dev->bufread + x + y * ll + dev->tw];
+		      lbuf[x * dev->bpp + y * ll + 2] =
+			dev->buf[dev->bufread + x + y * ll];
+		    }
 		}
 	    }
 	  /* avoids memcopy */
@@ -2033,7 +2119,7 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
   *len = length;
   dev->bufread += length;
   dev->read += length;
-  DBG (64, "sane_read %d bytes read\n", length);
+  DBG (64, "sane_read %ld bytes read\n", length);
 
   return SANE_STATUS_GOOD;
 
@@ -2092,7 +2178,7 @@ SANE_Status
 sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
 {
 
-  DBG (129, "unused arg: handle = %p, fd = %p\n", handle, fd);
+  DBG (129, "unused arg: handle = %p, fd = %p\n", handle, (void *) fd);
 
   DBG (2, "get_select_fd: not supported\n");
 
