@@ -876,7 +876,8 @@ process_backend_option (SANE_Handle device, int optnum, const char *optarg)
       exit (1);
     }
 
-  if ((opt->cap & SANE_CAP_AUTOMATIC) && strncasecmp (optarg, "auto", 4) == 0)
+  if ((opt->cap & SANE_CAP_AUTOMATIC) && optarg && 
+      strncasecmp (optarg, "auto", 4) == 0)
     {
       status = sane_control_option (device, optnum, SANE_ACTION_SET_AUTO,
 				    0, 0);
@@ -1015,6 +1016,7 @@ scan_it (void)
     "gray", "RGB", "red", "green", "blue"
   };
   SANE_Word total_bytes = 0, expected_bytes;
+  SANE_Int hang_over = -1;
 
   do
     {
@@ -1060,7 +1062,7 @@ scan_it (void)
 	    case SANE_FRAME_RED:
 	    case SANE_FRAME_GREEN:
 	    case SANE_FRAME_BLUE:
-	      assert ((parm.depth == 8) || (parm.depth == 16));
+	      assert (parm.depth == 8);
 	      must_buffer = 1;
 	      offset = parm.format - SANE_FRAME_RED;
 	      break;
@@ -1106,6 +1108,8 @@ scan_it (void)
 	      image.Bpp = 3;
 	      if (parm.format == SANE_FRAME_GRAY)
 		image.Bpp = 1;
+	      if (parm.depth == 16)
+		image.Bpp *= 2;
 	      image.x = image.width - 1;
 	      image.y = -1;
 	      if (!advance (&image))
@@ -1163,11 +1167,12 @@ scan_it (void)
 		  for (i = 0; i < len; ++i)
 		    {
 		      image.data[offset + i] = buffer[i];
-		      if ((offset + i) % 3 == 0 && !advance (&image))
-			{
-			  status = SANE_STATUS_NO_MEM;
-			  goto cleanup;
-			}
+		      if (image.Bpp == 3 || (offset + i) % 2 == 0)
+			if ((offset + i) % 3 == 0 && !advance (&image))
+			  {
+			    status = SANE_STATUS_NO_MEM;
+			    goto cleanup;
+			  }
 		    }
 		  offset += len;
 		  break;
@@ -1176,18 +1181,55 @@ scan_it (void)
 		  for (i = 0; i < len; ++i)
 		    {
 		      image.data[offset + i] = buffer[i];
-		      if (!advance (&image))
-			{
-			  status = SANE_STATUS_NO_MEM;
-			  goto cleanup;
-			}
+		      if (image.Bpp == 1 || (offset + i) % 2 == 0)
+			if (!advance (&image))
+			  {
+			    status = SANE_STATUS_NO_MEM;
+			    goto cleanup;
+			  }
 		    }
 		  offset += len;
 		  break;
 		}
 	    }
-	  else
-	    fwrite (buffer, 1, len, stdout);
+	  else /* ! must_buffer */
+  	    {
+	      if ((output_format == OUTPUT_TIFF) || (parm.depth != 16))
+		fwrite (buffer, 1, len, stdout);
+	      else
+	        {
+#if !defined(WORDS_BIGENDIAN)
+		  int i, start = 0;
+		  
+		  /* check if we have saved one byte from the last sane_read */
+		  if (hang_over > -1) 
+		    {
+		      if (len > 0)
+			{
+			  fwrite (buffer, 1, 1, stdout);
+			  buffer[0] = (SANE_Byte) hang_over;
+			  hang_over = -1;
+			  start = 1;
+			}
+		    }
+		  /* now do the byte-swapping */
+		  for (i = start; i < (len - 1); i += 2)
+		    {
+		      unsigned char LSB;
+		      LSB = buffer[i];
+		      buffer[i] = buffer[i+1];
+		      buffer[i+1] = LSB;
+		    }
+		  /* check if we have an odd number of bytes */
+		  if (((len - start) % 2) != 0)
+		    {
+		      hang_over = buffer [len - 1];
+		      len--;
+		    }
+#endif
+		  fwrite (buffer, 1, len, stdout);
+		}
+	    }
 
 	  if (verbose && parm.depth == 8)
 	    {
@@ -1210,7 +1252,23 @@ scan_it (void)
 				 parm.lines, parm.depth, resolution_value);
       else
 	write_pnm_header (parm.format, image.width, image.height, parm.depth);
-      fwrite (image.data, image.Bpp, image.height * image.width, stdout);
+      if ((output_format == OUTPUT_TIFF) || (image.Bpp == 1) 
+	  || (image.Bpp == 3))
+	fwrite (image.data, image.Bpp, image.height * image.width, stdout);
+      else /* image.Bpp == 2 or image.Bpp == 6 assumed */
+        {
+#if !defined(WORDS_BIGENDIAN)
+	  int i;
+	  for (i = 0; i < image.Bpp * image.height * image.width; i += 2)
+	    {
+	      unsigned char LSB;
+	      LSB = image.data[i];
+	      image.data[i] = image.data[i+1];
+	      image.data[i+1] = LSB;
+	    }
+#endif
+	  fwrite (image.data, image.Bpp, image.height * image.width, stdout);
+	}
     }
 
 cleanup:
@@ -1218,6 +1276,7 @@ cleanup:
   if (image.data)
     free (image.data);
 
+
   expected_bytes = parm.bytes_per_line * parm.lines * 
     ((parm.format == SANE_FRAME_RGB || parm.format == SANE_FRAME_GRAY)?1:3);
   if (parm.lines < 0)
@@ -1491,17 +1550,12 @@ main (int argc, char **argv)
 	      }
 	    else 
 	      {
-	        int i = 0, n = 0, maxarg = 8;
-	        char *percent;
-	        char *start, *fmt;
-	        const char **arglist = malloc(8 * sizeof(char**));
-	        if (arglist == 0)
-	          {
-	            fprintf(stderr, "%s: not enough memory\n", prog_name);
-	            exit(1);
-	          }
+	        int i = 0, int_arg = 0;
+	        char *percent, *start, *fmt;
+	        const char *text_arg = 0;
+	        char cc, ftype;
 	        
-	        fmt = malloc(strlen(optarg));
+	        fmt = malloc(strlen(optarg)+1);
 	        if (fmt == 0)
 	          {
 	            fprintf(stderr, "%s: not enough memory\n", prog_name);
@@ -1510,71 +1564,82 @@ main (int argc, char **argv)
 	        
 	        for (i = 0; device_list[i]; ++i)
 	          {
-	            n = 0;
 	            strcpy(fmt, optarg);
 	            start = fmt;
-	            while( (percent = strchr(start, '%')) )
+	            while(*start && (percent = strchr(start, '%')) )
 	              {
 	                percent++;
-  	                if (percent)
+  	                if (*percent)
 	                  {
-	                    if (n >= maxarg)
-	                      {
-	                        maxarg = 2 * maxarg;
-	                        arglist = realloc(arglist, maxarg * sizeof(char**));
-	                        if (arglist == 0)
-	                          {
-				    fprintf(stderr, "%s: not enough memory\n", 
-					    prog_name);
-		                    exit(1);
-	                          } 
-	                      }
 	                    switch (*percent)
 	                      {
 	                        case 'd':
-	                          arglist[n++] = device_list[i]->name;
-	                          *percent = 's';
+	                          text_arg = device_list[i]->name;
+	                          ftype = *percent = 's';
 	                          break;
 	                        case 'v':
-	                          arglist[n++] = device_list[i]->vendor;
-	                          *percent = 's';
+	                          text_arg = device_list[i]->vendor;
+	                          ftype = *percent = 's';
 	                          break;
 	                        case 'm':
-	                          arglist[n++] = device_list[i]->model;
-	                          *percent = 's';
+	                          text_arg = device_list[i]->model;
+	                          ftype = *percent = 's';
 	                          break;
 	                        case 't':
-	                          arglist[n++] = device_list[i]->type;
-	                          *percent = 's';
+	                          text_arg = device_list[i]->type;
+	                          ftype = *percent = 's';
 	                          break;
 	                        case 'i':
-	                          arglist[n++] = (char *) i;
+	                          int_arg = i;
+	                          ftype = 'i';
 	                          break;
 	                        case '%':
+	                          ftype = 0;
 	                          break;
 	                        default:
 	                          fprintf(stderr, "%s: unknown format specifier %%%c\n",
 					  prog_name, *percent);
 				  *percent = '%';
+				  ftype = 0;
 	                      }
 	                    percent++;
+	                    cc = *percent;
+	                    *percent = 0;
+	                    switch (ftype)
+	                      {
+	                        case 's':
+	                          printf(start, text_arg);
+	                          break;
+	                        case 'i':
+	                          printf(start, int_arg);
+	                          break;
+	                        case 0:
+	                          printf(start);
+	                          break;
+	                      }
+	                    *percent = cc;
+	                    start = percent;
+	                  }
+	                else
+	                  {
+	                    /* last char of the string is a '%', suppress it */
+	                    *start = 0;
+	                    break;
 	                  }
-	                start = percent;
 	              }
-	            vprintf(fmt, arglist);
-		    va_end(arglist);
+	            if (*start)
+	              printf(start);
 	          }
 	      }
-	        
 	    if (i == 0 && ch != 'f')
 	      printf ("\nNo scanners were identified. If you were expecting "
 		      "something different,\ncheck that the scanner is plugged "
 		      "in, turned on and detected by the\nsane-find-scanner tool "
 		      "(if appropriate). Please read the documentation\nwhich came "
 		      "with this software (README, FAQ, manpages).\n");
+
 	    if (defdevname)
 	      printf ("default device is `%s'\n", defdevname);
-
 	    exit (0);
 	  }
 
@@ -1763,7 +1828,6 @@ standard output.\n\
 	}
 
       free (full_optstring);
-
       for (index = 0; index < 2; ++index)
 	if (window[index])
 	  {
